/**
 * \brief Custom display on concentrics NeoPixel rings
 * \author Yann Pascaud
 * \version
 * 1. 11/08/2023 1er version
 * 
 * Copyright (c) 2023 yann pascaud
 * 
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program.
 * If not, see https://www.gnu.org/licenses/
 *
 */

#ifndef NeoRammstein_h
#define NeoRammstein_h

#ifdef ATTINY_CORE
#define USE_TINYNEOPIXEL_STATIC //!< \brief Automatic switch between tinyNeoPixel_Static / Adafruit_NeoPixel
#endif

#ifdef USE_TINYNEOPIXEL_STATIC
#include <tinyNeoPixel_Static.h>
#else
#include "Adafruit_NeoPixel.h"
#endif

#define NUM_LEDS 45 //!< \brief Total number of LED of the setup

//NeoFire 
#define MAIN_COLOR 80, 35, 00 //!< \brief NeoFire principal color
#define RANDOM_RED 1.0
#define RANDOM_GREEN 0.5
#define RANDOM_BLUE 0.5

//NeoRammstein
#define LEAFS_NUM 4     //!< \brief numbers of petals
#define LEAFS_SIZE 11   //!< \brief numbers of pixels per petals
#define CENTER_RING 0
#define INNER_RING 1
#define MIDDLE_RING 2
#define OUTTER_RING 3
#define BACK_RING 4

#ifdef USE_TINYNEOPIXEL_STATIC
class NeoRammstein : public tinyNeoPixel {
#else
class NeoRammstein : public Adafruit_NeoPixel {
#endif


public:

  #ifdef USE_TINYNEOPIXEL_STATIC
  NeoRammstein(uint16_t n, int16_t pin, neoPixelType type, uint8_t *pxl);
  void fill(uint32_t c = 0, uint16_t first = 0, uint16_t count = 0);
  static uint32_t ColorHSV(uint16_t hue, uint8_t sat = 255, uint8_t val = 255);
  #else
  NeoRammstein(uint16_t n, int16_t pin, neoPixelType type);
  #endif
 
  void setRingColor(uint8_t ring, uint32_t color); //!< \brief Color one ring
  // Séquences
  void startSequence();
  void drawFire(); //!< \brief NeoFire effect (https://www.arduinoslovakia.eu/blog/2015/9/simulacia-ohna?lang=en)
  void drawRainbow(); //!< \brief Concentric rainbows effect

private:
  uint32_t _elapsedTime=0, _waitTime=0;
  uint16_t _hue=0;
  void addColor(uint8_t position, uint32_t color);
  void substractColor(uint8_t position, uint32_t color);
  uint32_t blend(uint32_t color1, uint32_t color2);
  uint32_t substract(uint32_t color1, uint32_t color2);
  bool delay();

};

#endif
